import {connect}                                    from "react-redux";
import {Link}                                       from 'react-router-dom';
import {backendURL, actionAdById, actionNewComment} from "./Reducers";
import {useState}                                   from "react";

let AdImg = function({images}) {
    return (
        <div className="ad__img-container">
            <ul className="ad__list">
                {images && images.length != 0 ? 
                images.map(item => <li>
                                        <img className="ad__img"
                                             src={item.url ?
                                                `${backendURL}/${item.url}` :
                                                `${backendURL}/images/96ecc4298f6ffd0b0aaa957318fbeae5`} />
                                  </li>) :
                <li><img className="ad__img" src={`${backendURL}/images/96ecc4298f6ffd0b0aaa957318fbeae5`} /></li>}
            </ul>
        </div>
    );
};

let AdOwner = function({owner}) {
    return (
        <div className="ad__owner-container">
            <span>Владелец</span>
            <Link to={`/user/${owner?._id}`}>
                <img className="ad__owner-img" 
                     src={owner?.avatar ?
                          `${backendURL}/${owner?.avatar.url}` :
                          `${backendURL}/images/0c0a079786ed0b108acc21296ec2959f`} />
                {owner?.login}
            </Link>
            <Link to={`/user-ad/${owner?._id}`}>Другие объявления автора</Link>
        </div>
    );
};

let AdTags = function({tags}) {
    return (
        <ul className="ad__tags-list">
            {tags.map(text => <li>{text}</li>)}
        </ul>
    );
};

let AdInfo = function({address, createdAt, description, price, tags, title}) {
    return (
        <div className="ad__info-container">
            <span>Опубликовано: {(new Date(+createdAt)).toLocaleString()}</span>
            <h3>{title}</h3>
            <span>Цена: {price}</span>
            {tags && <AdTags  tags={tags}/>}
            <p>Описание:<br/>{description}</p>
        </div>
    );
};

let Comment = function({comment: {createdAt, owner, text, _id}}) {
    return (
        <li>
            <Link to={`/user/${owner?._id}`}>
                <img className="ad__comment-owner-img" 
                        src={owner?.avatar ?
                            `${backendURL}/${owner?.avatar.url}` :
                            `${backendURL}/images/0c0a079786ed0b108acc21296ec2959f`} />
                <span>{owner?.login}</span><br/>
            </Link>
            <p>{text}</p>
            <span>{(new Date(+createdAt)).toLocaleString()}</span>
        </li>
    );
};

let AdComments = function({comments}) {
    return (
        <div className="ad__comments-container">
            <h3>Комментарии</h3>
            <ul>
                {comments ? 
                    comments.map(comment => <Comment comment={comment} />) :
                    <p>К этому объявлению пока нет комментариев, станьте первым</p>} 
            </ul>
        </div>
    );
};

let AdNewComment = function({newComment, adId, answerToId}) {
    const [text, setText] = useState("");

    return (
        <div className="ad_new-comment-container">
            <textarea placeholder="Напишите ваш комментарий"
                      onChange={evt => setText(evt.target.value)} />
            <button onClick={() => newComment(text, adId, answerToId)}>Отправить</button>
        </div>
    );
};

const CAdNewComment = connect(null, {newComment: actionNewComment})(AdNewComment);

let Advertisement = function({ad: {address, comments, createdAt, description, images, owner, price, tags, title, _id}}) {
    return (
        <section className="ad">
            <h2 className="visually-hidden">Объявление</h2>
            <AdImg images={images} />
            <AdOwner owner={owner} />
            <AdInfo address={address} createdAt={createdAt} description={description} price={price} tags={tags} title={title} />
            <AdComments comments={comments}/>
            <CAdNewComment adId={_id} answerToId={comments && comments[comments?.length - 1]?._id} />
        </section>
    );
};

const CAdvertisement = connect(state => ({ad: state.promise?.adById?.payload || {}}))(Advertisement);

let AdPage = function({match: {params: {_id}}, adById}) {
    adById(_id);

    return (
        <CAdvertisement />
    );
};

const CAdPage = connect(null, {adById: actionAdById})(AdPage);

export default CAdPage;