import {Route, Switch, Redirect} from 'react-router-dom';
import CMainPage                 from "./MainPage";
import CLogin                    from "./Login";
import CPasswordChange           from "./PasswordChange";
import CAdPage                   from "./AdPage";
import CNewAd                    from "./NewAd";
import CProfile                  from "./Profile";
import { connect }               from 'react-redux';

const ProtectedRoute = function({roles=[], fallback="/login", component, auth, ...routeProps}) {
    const WrapperComponent = (renderProps) => {
        const C = component;
        if(!roles.filter(item => item.includes([auth])).length) {
            return <Redirect to={fallback} />
        };
        return <C {...renderProps} />
    }
    return <Route {...routeProps} component={WrapperComponent}/>
};

const CProtectedRoute = connect(state => ({auth: state.auth?.payload?.sub?.acl[1] || "anon"}))(ProtectedRoute);

let Main = function({authId}) {
    return (
        <main className="main">
            <h1 className="visually-hidden">Сервис для объявлений по купле/продажи товаров и услуг</h1>
            <Switch>
                <Route path="/"                exact component={CMainPage} />
                <Route path="/user-ad/:_id"    exact component={CMainPage} />
                <Route path="/password-change" exact component={CPasswordChange} />
                <CProtectedRoute roles={["anon"]} fallback={`/user/${authId}`} path="/login" component={CLogin} />
                <CProtectedRoute roles={["user"]} fallback="/login" path="/ad/:_id" component={CAdPage} />
                <CProtectedRoute roles={["user"]} fallback="/login" path="/new-ad" component={CNewAd} />
                <CProtectedRoute roles={["user"]} fallback="/login" path="/user/:_id" component={CProfile} />
            </Switch>
        </main>
    );
};

const CMain = connect(state => ({authId: state.auth?.payload?.sub?.acl[0]}))(Main);

export default CMain;