import {connect}                                          from "react-redux";
import {useDropzone}                                      from 'react-dropzone';
import React, {useState, useEffect}                       from "react";
import {backendURL, actionUploadFiles, actionSaveAdState} from "./Reducers";
import {sortableContainer, sortableElement}               from 'react-sortable-hoc';
import {arrayMoveImmutable}                               from 'array-move';
import Select                                             from 'react-select'

function Dropzone({uploadFiles}) {
    const {acceptedFiles, getRootProps, getInputProps} = useDropzone();

    useEffect(() => {
        if(acceptedFiles.length) {
            uploadFiles(acceptedFiles)
        }
    }, [acceptedFiles])

    return (
        <section className="container">
            <div {...getRootProps({className: 'dropzone'})}>
                <input name="photo" {...getInputProps()} />
                <p>Drag 'n' drop some files here, or click to select files</p>
            </div>
        </section>
    );
};

const CDropzone = connect(null, {uploadFiles: actionUploadFiles})(Dropzone);

const SortableContainer = sortableContainer(({children}) => {
    return (
        <ul className="new-ad__sort-list">{children}</ul>
    );
});

const SortableItem = sortableElement(({url}) => {
    return (
      <li>
        <img className="new-ad__sort-img" src={`${backendURL}/${url}`} />
      </li>
    );
  });

let NewAd = function({uploadFiles, onSave}) {
    const [state, setState] = useState({images: []});

    useEffect(() => {
        if(uploadFiles?.status == "FULFILLED") {
            setState({...state, images: [...state.images, ...uploadFiles?.payload]});
        };
    }, [uploadFiles]);

    let onSortEnd = ({oldIndex, newIndex}) => {
        setState(({images}) => {
            return {...state, images: arrayMoveImmutable(images, oldIndex, newIndex)}
        });
    };

    const selectTags = [
        { value: 'Детский мир', label: 'Детский мир' },
        { value: 'Недвижимость', label: 'Недвижимость' },
        { value: 'Авто', label: 'Авто' },
        { value: 'Запчасти для транспорта', label: 'Запчасти для транспорта' },
        { value: 'Работа', label: 'Работа' },
        { value: 'Животные', label: 'Животные' },
        { value: 'Дом и сад', label: 'Дом и сад' },
        { value: 'Электроника', label: 'Электроника' },
        { value: 'Хобби, отдых и спорт', label: 'Хобби, отдых и спорт' },
        { value: 'Обмен', label: 'Обмен' },
        { value: 'Медицина', label: 'Медицина' }
    ]

    return (
        <section className="new-ad">
            <h2 className="new-ad__title">Создать объявление</h2>
            <div className="new-ad__dropzone-container">
                <CDropzone />
                <SortableContainer onSortEnd={onSortEnd}>
                    {state.images.map((item, index) => <SortableItem index={index} url={item.url} />)}
                </SortableContainer>
            </div>
            <div className="new-ad__ad-info-container">
                <label>Оглавление<br/>
                    <input type="text"
                           placeholder="Введите оглавление"
                           onChange={(evt) => setState({...state, title: evt.target.value})} />
                </label><br/>
                <label>Описание<br/>
                    <textarea placeholder="Введите описание"
                              onChange={(evt) => setState({...state, description: evt.target.value})} />
                </label><br/>
                <label>Выберите один или несколько категорий которые больше всего подходят вашему объявлению
                    <Select isMulti
                            name="tags"
                            options={selectTags}
                            onChange={(tags) => setState({...state, tags: tags.map(tag => tag.label)})} />
                </label><br/>
                <label>Введите ваш адрес
                    <input type="text"
                           placeholder="Введите ваш адрес"
                           onChange={(evt) => setState({...state, address: evt.target.value})} />
                </label><br/>
                <label>Цена:
                    <input type="number"
                           onChange={(evt) => setState({...state, price: +evt.target.value})} />
                </label>
                <button onClick={() => onSave(state)}>Создать</button>
            </div>
        </section>
    );
};

const CNewAd = connect(state => ({uploadFiles: state.promise?.uploadFiles}),
                                 {onSave: actionSaveAdState})(NewAd);

export default CNewAd;